/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.compiler;

import org.eclipse.jdt.core.compiler.IProblem;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.ICompilerRequestor;

/**
 * The class that receives the result of a compilation. It could be an error
 * or a correct compilation. If an error is returned the errors are saved
 * otherwise the resulting classes are loaded into a custom class loader
 * and the Class is stored in a property (that can be requested).
 * @author ricky
 */
public class CompilerRequestor implements ICompilerRequestor {

    /**
     * The class name of the current java source file.
     */
    private String className = null;
    
    /**
     * The class loader used in the compilation.
     */
    private ClassLoader classLoader = null; 
    
    /**
     * The property that holds the main compiled class.
     */
    private Class resultClass = null;
    
    /**
     * Errors.
     */
    private StringBuilder errors = null;
    
    /**
     * Constructor via arguments.
     * @param className The class name being compiled.
     * @param classLoader The class loader used in the compilation.
     */
    public CompilerRequestor(String className, ClassLoader classLoader) {
        this.className = className;
        this.classLoader = classLoader;
        this.errors = new StringBuilder();
    }
    
    /**
     * Helper method to check if the compilation was ok.
     * @return true if compilation ok, false if errors
     */
    public boolean isSuccess() {
        return resultClass != null;
    }
    
    /**
     * Getter of the class in case of ok.
     * @return The compiled class
     */
    public Class getResultClass() {
        assert isSuccess();
        return resultClass;
    }
    
    /**
     * Return the errors.
     * @return The errors
     */
    public String getErrors() {
        return errors.toString();
    }
    
    /**
     * Create the class or the errors.
     * @param result The compilation result
     */
    @Override
    public void acceptResult(CompilationResult result) {
        if (result.hasErrors()) {
            IProblem[] problems = result.getProblems();
            for (IProblem problem : problems) {
                if (problem.isError()) {
                    errors.append(new String(problem.getOriginatingFileName()));
                    errors.append(":");
                    errors.append(problem.getSourceLineNumber());
                    errors.append(" ");
                    errors.append(problem.getMessage());
                    errors.append(System.getProperty("line.separator"));
                }
            }
        } else {
            try {
                CustomClassLoader custom = new CustomClassLoader(classLoader);
                custom.loadThisClass(result.getClassFiles());
                resultClass = custom.loadClass(className);
            } catch (ClassNotFoundException e) {
                throw new IllegalStateException(e);
            }
        }
    }
}
