/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package es.rickyepoderi.compiler.jsp;

import java.io.ByteArrayInputStream;
import java.nio.charset.Charset;
import javax.servlet.ServletContext;
import org.apache.jasper.JspCompilationContext;
import org.apache.jasper.Options;
import org.apache.jasper.compiler.JspRuntimeContext;
import org.apache.jasper.servlet.JspServletWrapper;

/**
 * A compilation context needed for the JSP compilation in memory. This class 
 * extends the common JspCompiletionContext of the jasper implementation.
 * 
 * @author ricky
 */
public class MemoryJspCompilationContext extends JspCompilationContext {

    /**
     * The jsp source that is going to be compiled.
     */
    String jspSource;
    
    /**
     * Constructor using parent arguments.
     * @param jspUri The jsp uri
     * @param jspSource The jsp source
     * @param options options used for compilation
     * @param context The servlet context
     * @param jsw The Jsp wrapper
     * @param rctxt The runtime context
     */
    public MemoryJspCompilationContext(String jspUri, String jspSource, Options options, 
            ServletContext context, JspServletWrapper jsw, JspRuntimeContext rctxt) {
        super(jspUri, options, context, jsw, rctxt);
        this.jspSource = jspSource;
    }
    
    /**
     * Method that override the normal processing and if the JSP requested is 
     * the one that is going to be compiled the jspSource property is returned.
     * @param res The string to search
     * @return The input stream of the resource
     */
    @Override
    public java.io.InputStream getResourceAsStream(String res) {
        if (this.getJspFile().equals(res)) {
            return new ByteArrayInputStream(jspSource.getBytes(Charset.forName("UTF-8")));
        } else {
            return super.getResourceAsStream(res);
        }
    }

    /**
     * Method that overrides the modified time for the current JSP. If the 
     * modified time is requested for this JSP current time is returned.
     * @param resource
     * @return 
     */
    @Override
    public Long getLastModified(String resource) {
        if (this.getJspFile().equals(resource)) {
            return System.currentTimeMillis();
        } else {
            return super.getLastModified(resource);
        }
    }

    /**
     * In order to avoid compilation files the checkOutputDir is disabled.
     */
    @Override
    public void checkOutputDir() {
        // NOOP cos everything is managed in memory
    }
    
    /**
     * In order to avoid the output dir empty String is returned.
     * @return empty string
     */
    @Override
    public String getOutputDir() {
        // managed in memory, no output directory
        return "";
    }
    
}
