/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package es.rickyepoderi.jsf;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLStreamHandlerFactory;
import javax.faces.application.ResourceHandler;
import javax.faces.application.ResourceHandlerWrapper;
import javax.faces.application.ViewResource;
import javax.faces.context.FacesContext;

/**
 * <p>The MemoryResourceHadler is the handler to provide my own XHTML templates.
 * In this case the templates will be recovered (as in all the other cases) from
 * the context of the app. The MemoryResourceHadler extends the ResourceHandlerWrapper
 * and only implements the createViewResource method. This method is executed
 * by JSF in order to find a Facelet template.</p>
 * 
 * <p>A ResourceHandler is based on URL implementation. So in order to create 
 * my memory repository a new <em>mem://</em> protocol is used. This URL 
 * implementation should be added inside the container. In Tomcat is done
 * using the TomcatURLStreamHandlerFactory.addUserFactory method.</p>
 * 
 * <p>The <em>mem://</em> protocol in the PoC is really simple. The path is 
 * searched in the context of the application from a specified starting point
 * (/WEB-INF/classes/es/rickyepoderi/samples/facelets). this is just an example,
 * in a real scenario the protocol should search the resource from a real
 * repository.</p>
 * 
 * @author ricky
 */
public class MemoryResourceHandler extends ResourceHandlerWrapper {

    /**
     * In Tomcat to include a custom URL there is a addUserFactory method.
     * This static part detects if we are inside a Tomcat container and
     * injects the <em>Memory</rm> Handler.
     */
    static {
        System.err.println("================> Adding MemoryStreamHandlerFactory");
        try {
            Class tomcathfc = Class.forName("org.apache.catalina.webresources.TomcatURLStreamHandlerFactory");
            if (tomcathfc != null) {
                try {
                    Method m = tomcathfc.getMethod("getInstance");
                    Object tomcathfo = m.invoke(null);
                    System.err.println(tomcathfo);
                    m = tomcathfo.getClass().getMethod("addUserFactory", URLStreamHandlerFactory.class);
                    m.invoke(tomcathfo, new MemoryStreamHandlerFactory());
                } catch (IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
                    e.printStackTrace();
                }
            }
        } catch (ClassNotFoundException e) {
            // NOOP
        }
    }
    
    /**
     * The warpper or original resource handler.
     */
    private final ResourceHandler wrapped;

    /**
     * Public constructor.
     * @param wrapped The wrapped resource handler
     */
    public MemoryResourceHandler(final ResourceHandler wrapped) {
        this.wrapped = wrapped;
    }

    /**
     * Return the wraped resource handler.
     * @return the wrapped resource handler.
     */
    @Override
    public ResourceHandler getWrapped() {
        return wrapped;
    }
    
    /**
     * This method will be used to return the memory templates. The method first
     * searched the resource inside the wrapped resource handler and, if the
     * resource is not found, it is searched inside memory. This method does not
     * check if the resource really exists, it just return the URL.
     * @param context The  faces context
     * @param resourceName The name of the resource to search
     * @return The ViewResource or null
     */
    @Override
    public ViewResource createViewResource(FacesContext context, String resourceName) {
        System.err.println("MemoryResourceHandler.createViewResource() " + resourceName);
        ViewResource vres = wrapped.createViewResource(context, resourceName);
        if (vres == null) {
            try {
                // TODO: this method should return null if the resource
                //       does not exists. Now it is always returnen the URL.
                //       It doesn't matter if the resource really exists or not
                vres = new MemoryViewResource(new URL("mem://" + resourceName));
            } catch(MalformedURLException e) {
                e.printStackTrace();
            }
        }
        return vres;
    }

}
