/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package es.rickyepoderi.servlet;

import es.rickyepoderi.compiler.DefaultCompilerOptions;
import es.rickyepoderi.compiler.jsp.MemoryCompiler;
import es.rickyepoderi.compiler.jsp.MemoryJspCompilationContext;
import es.rickyepoderi.compiler.SimpleCompiler;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.jasper.JspCompilationContext;
import org.apache.jasper.compiler.JspRuntimeContext;

/**
 * <p>The compiler servlet tries to compile another thing (first servlet, then
 * JSP) and forward the processing to the new class. Now the real JSP or servlet
 * is read from the same WAR file (packaged in samples directory) but this is
 * just for the PoC (in a real implementation the source should be retrieved
 * from ddbb or another repository). There are two parameters:</p>
 * 
 * <ul>
 * <li>name: The name of the servlet or JSP inside the samples directory.</li>
 * <li>type: <em>jsp</em> or <em>servlet</em>.</li>
 * </ul>
 * 
 * @author ricky
 */
public class CompilerServlet extends HttpServlet {

    /**
     * Method to compile a servlet java file in-line and forward the processing
     * to it. The servlet source is read from 
     * <em>/WEB-INF/classes/es/rickyepoderi/samples/servlet/</em> directory.
     * 
     * @param request The servlet request
     * @param response The servlet response
     * @throws ServletException Some exception
     * @throws IOException Some exception
     */
    protected void processCompileServlet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try (InputStream in = this.getServletContext().getResourceAsStream(
                "/WEB-INF/classes/es/rickyepoderi/samples/servlet/" + request.getParameter("name") + ".txt")) {
            SimpleCompiler compiler = new SimpleCompiler(Thread.currentThread().getContextClassLoader(), 
                    new DefaultCompilerOptions());
            Class clazz = compiler.compile("es.rickyepoderi.samples.servlet." + request.getParameter("name"), new InputStreamReader(in));
            HttpServlet servlet = (HttpServlet) clazz.newInstance();
            servlet.init(this.getServletConfig());
            servlet.service(request, response);
        } catch (InstantiationException | IllegalAccessException e) {
            throw new ServletException(e);
        }
    }
    
    /**
     * Method to read the JSP file into a String. The JSp is read from the
     * directory <em>/WEB-INF/classes/es/rickyepoderi/samples/jsp/</em>.
     * @param name The name of the JSP
     * @return The JSP in a String format
     * @throws IOException Some error
     */
    private String readJspFile(String name) throws IOException {
        try (BufferedReader in = new BufferedReader(new InputStreamReader(this.getServletContext().getResourceAsStream(
                "/WEB-INF/classes/es/rickyepoderi/samples/jsp/" + name + ".jsp")))) {
            String line = null;
            StringBuilder stringBuilder = new StringBuilder();
            String ls = System.getProperty("line.separator");
            while ((line = in.readLine()) != null) {
                stringBuilder.append(line);
                stringBuilder.append(ls);
            }
            return stringBuilder.toString();
        }
    }
    
    /**
     * Method to parse and compile a JSP using my in-memory implementation.
     * @param request The servlet request
     * @param response The servlet response
     * @throws ServletException Some error
     * @throws IOException  Some error
     */
    protected void processCompileJsp(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            String jspSource = readJspFile(request.getParameter("name"));
            DefaultCompilerOptions opts = new DefaultCompilerOptions(this.getServletContext());            
            JspRuntimeContext rtctx = new JspRuntimeContext(this.getServletContext(), opts);
            JspCompilationContext clctxt = new MemoryJspCompilationContext(
                    "/" + request.getParameter("name") + ".jsp", jspSource, opts, this.getServletContext(), null, rtctx);
            org.apache.jasper.compiler.Compiler clc = clctxt.createCompiler();
            clc.compile(true, true);
            Class clazz = ((MemoryCompiler) clc).getGeneratedClass();
            if (javax.servlet.jsp.JspFactory.getDefaultFactory() == null) {
                // initialize in case Tomcat has not done yet
                javax.servlet.jsp.JspFactory.setDefaultFactory(
                        new org.apache.jasper.runtime.JspFactoryImpl());
            }
            HttpServlet servlet = (HttpServlet) clazz.newInstance();
            servlet.init(this.getServletConfig());
            servlet.service(request, response);
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }
    
    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        if (request.getParameter("name") == null) {
            throw new ServletException("The parameter 'name' is compulsory");
        }
        if ("servlet".equalsIgnoreCase(request.getParameter("type"))) {
            processCompileServlet(request, response);
        } else if ("jsp".equalsIgnoreCase(request.getParameter("type"))) {
            processCompileJsp(request, response);
        } else {
            throw new ServletException("The parameter 'type' should be 'servlet' or 'jsp'");
        }
    }

    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }
    
}
